unit Digit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TBar = (brUp, brLeftUp, brRightUp, brMiddle, brLeftDown, brRightDown, brDown);
  TBars = set of TBar;
type
  TDigit = class(TGraphicControl)
  private
    FBars: TBars;
    FOnPen: TPen;
    FOffPen: TPen;
    FOnBrush: TBrush;
    FOffBrush: TBrush;
    FValue: char;
    FShift: integer;

    FOnMouseMove: TMouseMoveEvent;
    function GetBars: TBars;
    procedure SetBars(v: TBars);
    procedure SetOnPen(v: TPen);
    procedure SetOffPen(v: TPen);
    procedure SetOnBrush(v: TBrush);
    procedure SetOffBrush(v: TBrush);
    procedure OnChange(Sender: TObject);
    procedure SetValue(v: char);
    procedure SetShift(v: integer);
  protected
    x, y: array [TBar, 0..10] of integer;
    procedure WMMouseMove(var msg: TWMMouseMove); message WM_MouseMove;
  public
    procedure Paint; override;
    constructor Create(AOwner: TComponent); override;
  published
    property Bars: TBars read GetBars write SetBars;
    property OnPen: TPen read FOnPen write SetOnPen;
    property OffPen: TPen read FOffPen write SetOffPen;
    property OnBrush: TBrush read FOnBrush write SetOnBrush;
    property OffBrush: TBrush read FOffBrush write SetOffBrush;
    property Value: char read FValue write SetValue;
    property Shift: integer read FShift write SetShift;

    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
  end;

procedure Register;

implementation

procedure TDigit.WMMouseMove(var msg: TWMMouseMove);
begin
  if Assigned(FOnMouseMove)
    then FOnMouseMove(self, KeysToShiftState(msg.Keys),
      msg.XPos, msg.YPos);
end;

constructor TDigit.Create(AOwner: TComponent);
begin
  inherited;
  FOnPen := TPen.Create;
  FOffPen := TPen.Create;
  FOnBrush := TBrush.Create;
  FOffBrush := TBrush.Create;

  FOnPen.Color := clBlack;
  FOffPen.Color := clGray;
  FOnBrush.Color := clGray;
  FOffBrush.Color := clSilver;

  FOnPen.OnChange := OnChange;
  FOffPen.OnChange := OnChange;
  FOnBrush.OnChange := OnChange;
  FOffBrush.OnChange := OnChange;

  Width := 18;
  Height := 30;

  FShift := 2;
  FValue := #0;
  FBars := [];

  x[brUp,0] := 3;  y[brUp,0] := 0;
  x[brUp,1] := 14; y[brUp,1] := 0;
  x[brUp,2] := 15; y[brUp,2] := 1;
  x[brUp,3] := 12; y[brUp,3] := 4;
  x[brUp,4] := 5;  y[brUp,4] := 4;
  x[brUp,5] := 2;  y[brUp,5] := 1;

  x[brLeftUp,0] := 1; y[brLeftUp,0] := 2;
  x[brLeftUp,1] := 4; y[brLeftUp,1] := 5;
  x[brLeftUp,2] := 4; y[brLeftUp,2] := 10;
  x[brLeftUp,3] := 1; y[brLeftUp,3] := 13;
  x[brLeftUp,4] := 0; y[brLeftUp,4] := 12;
  x[brLeftUp,5] := 0; y[brLeftUp,5] := 3;

  x[brRightUp,0] := 16; y[brRightUp,0] := 2;
  x[brRightUp,1] := 17; y[brRightUp,1] := 3;
  x[brRightUp,2] := 17; y[brRightUp,2] := 12;
  x[brRightUp,3] := 16; y[brRightUp,3] := 13;
  x[brRightUp,4] := 13; y[brRightUp,4] := 10;
  x[brRightUp,5] := 13; y[brRightUp,5] := 5;

  x[brMiddle,0] := 4;  y[brMiddle,0] := 12;
  x[brMiddle,1] := 13; y[brMiddle,1] := 12;
  x[brMiddle,2] := 15; y[brMiddle,2] := 14;
  x[brMiddle,3] := 13; y[brMiddle,3] := 16;
  x[brMiddle,4] := 4;  y[brMiddle,4] := 16;
  x[brMiddle,5] := 2;  y[brMiddle,5] := 14;

  x[brLeftDown,0] := 1; y[brLeftDown,0] := 15;
  x[brLeftDown,1] := 4; y[brLeftDown,1] := 18;
  x[brLeftDown,2] := 4; y[brLeftDown,2] := 24;
  x[brLeftDown,3] := 1; y[brLeftDown,3] := 27;
  x[brLeftDown,4] := 0; y[brLeftDown,4] := 26;
  x[brLeftDown,5] := 0; y[brLeftDown,5] := 16;

  x[brRightDown,0] := 16; y[brRightDown,0] := 15;
  x[brRightDown,1] := 17; y[brRightDown,1] := 16;
  x[brRightDown,2] := 17; y[brRightDown,2] := 26;
  x[brRightDown,3] := 16; y[brRightDown,3] := 27;
  x[brRightDown,4] := 13; y[brRightDown,4] := 24;
  x[brRightDown,5] := 13; y[brRightDown,5] := 18;

  x[brDown,0] := 5;  y[brDown,0] := 25;
  x[brDown,1] := 12; y[brDown,1] := 25;
  x[brDown,2] := 15; y[brDown,2] := 28;
  x[brDown,3] := 14; y[brDown,3] := 29;
  x[brDown,4] := 3;  y[brDown,4] := 29;
  x[brDown,5] := 2;  y[brDown,5] := 28;
end;

procedure TDigit.Paint;
var
  XSc, YSc: single;
  bar: TBar;
  p: array [0..5] of TPoint;
  i: integer;
begin
  XSc := (Width - FShift) / 18;
  YSc := Height / 30;
  with Canvas do begin
    for bar := brUp to brDown do begin
      if bar in FBars then begin
        Pen := FOnPen;
        Brush := FOnBrush;
      end {then} else begin
        Pen := FOffPen;
        Brush := FOffBrush;
      end; {else}
      for i := 0 to 5 do
        p[i] := Point(round(x[bar,i] * XSc - FShift * (y[bar,i] / 30 - 1)), round(y[bar,i] * YSc));
      Polygon(p);
    end; {for}
  end;
end;

function TDigit.GetBars: TBars;
begin
  result := FBars;
end;

procedure TDigit.SetBars(v: TBars);
begin
  if FBars = v then Exit;
  FBars := v;
  FValue := #0;
  Repaint;
end;

procedure TDigit.SetOnPen(v: TPen);
begin
  if FOnPen = v then Exit;
  FOnPen := v;
  Repaint;
end;

procedure TDigit.SetOffPen(v: TPen);
begin
  if FOffPen = v then Exit;
  FOffPen := v;
  Repaint;
end;

procedure TDigit.SetOnBrush(v: TBrush);
begin
  if FOnBrush = v then Exit;
  FOnBrush := v;
  Repaint;
end;

procedure TDigit.SetOffBrush(v: TBrush);
begin
  if FOffBrush = v then Exit;
  FOffBrush := v;
  Repaint;
end;

procedure TDigit.OnChange(Sender: TObject);
begin
  Repaint;
end;

procedure TDigit.SetValue(v: char);
begin
  if FValue = v then Exit;
  FValue := v;
  case FValue of
    '0', 'D', 'O': FBars := [brUp, brLeftUp, brRightUp, brLeftDown, brRightDown, brDown];
    '1', '|', 'I', 'l': FBars := [brRightUp, brRightDown];
    '2': FBars := [brUp, brRightUp, brMiddle, brLeftDown, brDown];
    '3': FBars := [brUp, brRightUp, brMiddle, brRightDown, brDown];
    '4': FBars := [brLeftUp, brRightUp, brMiddle, brRightDown];
    '5', 's', 'S': FBars := [brUp, brLeftUp, brMiddle, brRightDown, brDown];
    '6': FBars := [brUp, brLeftUp, brMiddle, brLeftDown, brRightDown, brDown];
    '7': FBars := [brUp, brRightUp, brRightDown];
    '8', 'B': FBars := [brUp, brLeftUp, brRightUp, brMiddle, brLeftDown, brRightDown, brDown];
    '9': FBars := [brUp, brLeftUp, brRightUp, brMiddle, brRightDown, brDown];
    ' ': FBars := [];
    '_': FBars := [brDown];
    '-': FBars := [brMiddle];
    '~': FBars := [brUp];
    '[', '(': FBars := [brUp, brLeftUp, brLeftDown, brDown];
    ']', ')': FBars := [brUp, brRightUp, brRightDown, brDown];
    'A', 'R': FBars := [brUp, brLeftUp, brRightUp, brMiddle, brLeftDown, brRightDown];
    'b': FBars := [brLeftUp, brMiddle, brLeftDown, brRightDown, brDown];
    'c': FBars := [brMiddle, brLeftDown, brDown];
    'C': FBars := [brUp, brLeftUp, brLeftDown, brDown];
    'd': FBars := [brRightUp, brMiddle, brLeftDown, brRightDown, brDown];
    'e': FBars := [brUp, brLeftUp, brRightUp, brMiddle, brLeftDown, brDown];
    'E': FBars := [brUp, brLeftUp, brMiddle, brLeftDown, brDown];
    'f', 'F': FBars := [brUp, brLeftUp, brMiddle, brLeftDown];
    'h': FBars := [brLeftUp, brMiddle, brLeftDown, brRightDown];
    'H': FBars := [brLeftUp, brRightUp, brMiddle, brLeftDown, brRightDown];
    'i': FBars := [brRightDown];
    'L': FBars := [brLeftUp, brLeftDown, brDown];
    'o': FBars := [brMiddle, brLeftDown, brRightDown, brDown];
    'P', 'p': FBars := [brUp, brLeftUp, brRightUp, brMiddle, brLeftDown];
    'q': FBars := [brUp, brLeftUp, brRightUp, brMiddle, brRightDown];
    'r': FBars := [brMiddle, brLeftDown];
    'u': FBars := [brLeftDown, brRightDown, brDown];
    'U': FBars := [brLeftUp, brRightUp, brLeftDown, brRightDown, brDown];
    else FValue := #0;
  end; {case}
  Repaint;
end;

procedure TDigit.SetShift(v: integer);
begin
  if FShift = v then Exit;
  FShift := v;
  Repaint;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TDigit]);
end;

end.
